#ephy-status-frame {
  border-style: solid;
  border-width: 1;
  padding: 4;
}

#ephy-tab-close-button {
  -GtkButton-default-border: 0;
  -GtkButton-default-outside-border: 0;
  -GtkButton-inner-border: 0;
  -GtkWidget-focus-padding: 0;
  -GtkWidget-focus-line-width: 0;
  margin: 0;
  padding: 0;
}

